local LastBagID = NUM_BANKBAGSLOTS + 4;
local SetSize = {5,NUM_BANKBAGSLOTS + 1};
local MaxCont = {1,1};
local NumCont = {};

BaudBag_Debug = false;

--Backgrounds - party, tutorial, slider


local function DebugMsg(msg)
  if BaudBag_Debug then
    DEFAULT_CHAT_FRAME:AddMessage(msg);
  end
end


function BaudBag_OnLoad()
  local SubBag, Container;
  this:RegisterEvent("BAG_UPDATE");
  this:RegisterEvent("BAG_CLOSED");
  this:RegisterEvent("PLAYERBANKSLOTS_CHANGED");
  for BagSet = 1, 2 do
    --The first container from each set is different and is created in the XML
    Container = getglobal("BBCont"..BagSet.."_1");
    Container.BagSet = BagSet;
    Container:SetID(1);
  end

  --The first bag from the bank is unique and is created in the XML
  DebugMsg("Creating sub bags.");
  for Bag = -1, LastBagID do
    if(Bag == -1)then
      SubBag = getglobal("BaudBagSubBag"..Bag);
    else
      SubBag = CreateFrame("Button","BaudBagSubBag"..Bag,nil,"BaudBagSubBagTemplate");
    end
    SubBag:SetID(Bag);
    SubBag.BagSet = (Bag <= 4)and(Bag >= 0)and 1 or 2;
    SubBag:SetParent("BBCont"..SubBag.BagSet.."_1");
  end
end


function BaudBag_OnEvent()
  local BagSet;
  if(event=="BAG_UPDATE")or(event=="BAG_CLOSED")then
    BagSet = (arg1 >= 0)and(arg1 <= 4)and 1 or 2;
  elseif(event=="PLAYERBANKSLOTS_CHANGED")then
    BaudBagUpdateColors(getglobal("BaudBagSubBag-1"));
    BagSet = 2;
  else
    return;
  end
  local Container = getglobal("BBCont"..BagSet.."_1");
  if not Container:IsShown()then
    return;
  end
  Container.UpdateSlots = true;
end


local DropDownContainer, DropDownBagSet;

function BaudBagContainerDropDown_Show()
  local Container = this:GetParent();
  DropDownContainer = Container:GetID();
  DropDownBagSet = Container.BagSet;
  ToggleDropDownMenu(1, nil, BaudBagContainerDropDown, this:GetName(), 0, 0);
end


function BaudBagContainerDropDown_OnLoad()
  UIDropDownMenu_Initialize(this, BaudBagContainerDropDown_Initialize, "MENU");
end


local function ToggleContainerLock()
  BaudBag_Cfg[DropDownBagSet][DropDownContainer].Locked = not BaudBag_Cfg[DropDownBagSet][DropDownContainer].Locked;
end


local function ShowContainerOptions()
  BaudBagOptionsSelectContainer(DropDownBagSet,DropDownContainer);
  BaudBagOptionsFrame:Show();
end


function BaudBagContainerDropDown_Initialize()
  local info = UIDropDownMenu_CreateInfo();
  
  info.text = not(DropDownBagSet and BaudBag_Cfg[DropDownBagSet][DropDownContainer].Locked)and "Lock Position" or "Unlock Position";
  info.func = ToggleContainerLock;
  UIDropDownMenu_AddButton(info);

  info.text = "Options";
  info.func = ShowContainerOptions;
  UIDropDownMenu_AddButton(info);
end


--This function updates misc. options for a bag, such as the name and scale
function BaudUpdateContainerData(BagSet, ContNum)
  local Container = getglobal("BBCont"..BagSet.."_"..ContNum);
  DebugMsg("Updating container data: "..Container:GetName());
  getglobal(Container:GetName().."Name"):SetText(BaudBag_Cfg[BagSet][ContNum].Name or "");
  if not BaudBag_Cfg[BagSet][ContNum].Coords then
    BaudBagContainerSaveCoords(Container);
  end
  local Scale = BaudBag_Cfg[BagSet][ContNum].Scale / 100;
  Container:SetScale(Scale);
  Container:ClearAllPoints();
  local X, Y = unpack(BaudBag_Cfg[BagSet][ContNum].Coords);
  Container:SetPoint("CENTER",UIParent,"BOTTOMLEFT",(X / Scale), (Y / Scale));
end


local TextureFile, TextureWidth, TextureHeight, TextureParent;

local function GetTexturePiece(Name, MinX, MaxX, MinY, MaxY, Layer)
  local Texture = getglobal(TextureParent:GetName()..Name);
  if not Texture then
    Texture = TextureParent:CreateTexture(TextureParent:GetName()..Name);
    Texture:SetDrawLayer(Layer);
  end
  Texture:ClearAllPoints();
  Texture:SetTexture(TextureFile);
  Texture:SetTexCoord(MinX / TextureWidth, (MaxX + 1) / TextureWidth, MinY / TextureHeight, (MaxY + 1) / TextureHeight);
  Texture:SetWidth(MaxX - MinX + 1);
  Texture:SetHeight(MaxY - MinY + 1);
  return Texture;
end


function BaudBagUpdateBackground(Container)
  local Background = BaudBag_Cfg[Container.BagSet][Container:GetID()].Background;
  local Backdrop = getglobal(Container:GetName().."Backdrop");
  Backdrop:SetFrameLevel(Container:GetFrameLevel());
  local Left, Right, Top, Bottom;
  --This shifts the name of the bank frame over to make room for the extra button
  local ShiftName = (Container~=BBCont2_1)and 0 or 30;
  
  if(Background <= 3)then
    Left, Right, Top, Bottom = 10, 10, 26 + 18, 7;
    local Parent = Backdrop:GetName().."Textures";
    TextureParent = getglobal(Parent);
    TextureParent:SetFrameLevel(Container:GetFrameLevel());
    local Texture;
    
    TextureFile = "Interface\\ContainerFrame\\UI-Bag-Components";
    if(Background==2)then
      TextureFile = TextureFile.."-Bank";
    elseif(Background==3)then
      TextureFile = TextureFile.."-Keyring";
    end
    TextureWidth, TextureHeight = 256, 512;
    
    Texture = GetTexturePiece("BottomLeft",72,79,169,177,"ARTWORK");
    Texture:SetPoint("BOTTOMLEFT");
      
    Texture = GetTexturePiece("BottomRight",247,252,172,177,"ARTWORK");
    Texture:SetPoint("BOTTOMRIGHT");
      
    Texture = GetTexturePiece("Bottom",79,247,173,177,"ARTWORK");
    Texture:SetPoint("BOTTOM");
    Texture:SetPoint("LEFT",Parent.."BottomLeft","RIGHT");
    Texture:SetPoint("RIGHT",Parent.."BottomRight","LEFT");
    
    Texture = GetTexturePiece("TopLeft", 65, 116, 1, 49,"ARTWORK");
    Texture:SetPoint("TOPLEFT", -7, 4);
    
    Texture = GetTexturePiece("TopRight", 223, 252, 5, 49,"ARTWORK");
    Texture:SetPoint("TOPRIGHT");
    
    Texture = GetTexturePiece("Top", 116, 223, 5, 49,"ARTWORK");
    Texture:SetPoint("TOP");
    Texture:SetPoint("RIGHT",Parent.."TopRight","LEFT");
    Texture:SetPoint("LEFT",Parent.."TopLeft","RIGHT");
    
    Texture = GetTexturePiece("Left",72,76,182,432,"ARTWORK");
    Texture:SetPoint("LEFT");
    Texture:SetPoint("BOTTOM",Parent.."BottomLeft","TOP");
    Texture:SetPoint("TOP",Parent.."TopLeft","BOTTOM");
    
    Texture = GetTexturePiece("Right",248,252,182,432,"ARTWORK");
    Texture:SetPoint("RIGHT");
    Texture:SetPoint("BOTTOM",Parent.."BottomRight","TOP");
    Texture:SetPoint("TOP",Parent.."TopRight","BOTTOM");
  
    Texture = GetTexturePiece("BottomFill",112,248,30,49,"BACKGROUND");
    Texture:SetPoint("BOTTOM",Container,"BOTTOM",0,-5);
    Texture:SetPoint("RIGHT",Parent.."Right","LEFT");
    Texture:SetPoint("LEFT",Parent.."Left","RIGHT");
    Texture:SetHeight(45);
    
    --Width is 42, Height is 41
    local Cols = BaudBag_Cfg[Container.BagSet][Container:GetID()].Columns;
    local Row = 1;
--[[    local TopRow = Cols - mod(Container.Slots - 1, Cols) - 1;
    if(TopRow == 0)then
      Top = Top + 9;
    end]]
    local Col = 0;--TopRow;
    local OffsetX, OffsetY;
    for Slot = 1, Container.Slots do
      Col = Col + 1;
      if(Col > Cols)then
        Col = 1;
        Row = Row + 1;
      end
--[[      if(Row == 1)then --and(Col <= 1)then
        Texture = GetTexturePiece("Slot"..Slot,76,121,49,90,"Background");
        OffsetX, OffsetY = -2, 2;
        Texture = GetTexturePiece("Slot"..Slot,118,164,50,89,"BORDER");
        OffsetX, OffsetY = -2, 2;
      else]]
        Texture = GetTexturePiece("Slot"..Slot,118,164,215,256,"BORDER");
        OffsetX, OffsetY = -2, 0;
--      end
      Texture:SetPoint("TOPLEFT",Container,"TOPLEFT",(Col - 1) * 42 + OffsetX - 3, (Row - 1) * -41 + 2 - OffsetY);
      Texture:Show();
    end
    if(Container.Slots > (TextureParent.Slots or 0))then
      TextureParent.Slots = Container.Slots;
    else
      --Hide extra slot textures
      for Slot = (Container.Slots + 1), TextureParent.Slots do
        getglobal(TextureParent:GetName().."Slot"..Slot):Hide();
      end
    end
    
    --Adds the box for the money/slot indicators
    if(Container:GetID()==1)then
      Bottom = Bottom + 20;
      TextureFile = "Interface\\ContainerFrame\\UI-BackpackBackground.blp";
      TextureWidth, TextureHeight = 256, 256;

      Texture = GetTexturePiece("BottomFillLeft",80,84,213,231,"BACKGROUND");
      Texture:SetPoint("LEFT",Parent.."Left","RIGHT");
      Texture:SetPoint("BOTTOM",Parent.."Bottom","TOP",0,-2);
      
      Texture = GetTexturePiece("BottomFillRight",240,244,213,231,"BACKGROUND");
      Texture:SetPoint("RIGHT",Parent.."Right","LEFT");
      Texture:SetPoint("BOTTOM",Parent.."Bottom","TOP",0,-2);

      Texture = GetTexturePiece("BottomFillCenter",85,239,213,231,"BACKGROUND");
      Texture:SetPoint("LEFT",Parent.."BottomFillLeft","RIGHT");
      Texture:SetPoint("RIGHT",Parent.."BottomFillRight","LEFT");
    end
    
    Texture = getglobal(Parent.."Bag");
    if not Texture then
      Texture = TextureParent:CreateTexture(Parent.."Bag");
      Texture:SetWidth(40);
      Texture:SetHeight(40);
      Texture:ClearAllPoints();
      Texture:SetPoint("TOPLEFT",Parent.."TopLeft","TOPLEFT",3,-3);
      Texture:SetDrawLayer("BACKGROUND");
    end
    local Icon;
    local BagID = Container.Bags[1]:GetID();
    if(BagID <= 0)then
      Icon = (BagID == 0)and "Interface\\Buttons\\Button-Backpack-Up" or "Interface\\Icons\\INV_Box_02";
    else
      Icon = GetInventoryItemTexture("player",ContainerIDToInventoryID(BagID))or "Interface\\Icons\\INV_Misc_QuestionMark";
    end
    SetPortraitToTexture(Texture,Icon);
 
    Backdrop:SetBackdrop(nil);

    getglobal(Container:GetName().."Name"):SetPoint("TOPLEFT",Backdrop,"TOPLEFT",(40 + ShiftName),-7);
    getglobal(Container:GetName().."CloseButton"):SetPoint("TOPRIGHT",Backdrop,"TOPRIGHT",3,3);
    TextureParent:Show();
    if(Container:GetID()==1)then
      getglobal(Container:GetName().."Slots"):SetPoint("BOTTOMLEFT",Backdrop,"BOTTOMLEFT",12,7);
      getglobal(Container:GetName().."MoneyFrame"):SetPoint("BOTTOMRIGHT",Backdrop,"BOTTOMRIGHT",0,6);
    end
  else
    Left, Right, Top, Bottom = 8, 8, 28, 8;
    getglobal(Backdrop:GetName().."Textures"):Hide();
    
    getglobal(Container:GetName().."Name"):SetPoint("TOPLEFT",(2 + ShiftName),18);
    getglobal(Container:GetName().."CloseButton"):SetPoint("TOPRIGHT",8,28);
    if(Container:GetID()==1)then
      getglobal(Container:GetName().."Slots"):SetPoint("BOTTOMLEFT",2,-17);
      getglobal(Container:GetName().."MoneyFrame"):SetPoint("BOTTOMRIGHT",-8,-18);
      Bottom = Bottom + 18;
    end
    
    if(Background==5)then
      Backdrop:SetBackdrop({
        bgFile = "Interface\\Buttons\\WHITE8X8",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", 
        tile = true, tileSize = 8, edgeSize = 32, 
        insets = { left = 11, right = 12, top = 12, bottom = 11 }
      });
      Left, Right, Top, Bottom = Left+8, Right+8, Top+8, Bottom+8;
      Backdrop:SetBackdropColor(0.1,0.1,0.1,1);
    else
      Backdrop:SetBackdrop({
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
        tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 5, right = 5, top = 5, bottom = 5 }
      });
      Backdrop:SetBackdropColor(0,0,0,1);
    end
  end
  getglobal(Container:GetName().."Name"):SetPoint("RIGHT",Container:GetName().."MenuButton","LEFT");
  
  Backdrop:ClearAllPoints();
  Backdrop:SetPoint("TOPLEFT",-Left,Top);
  Backdrop:SetPoint("BOTTOMRIGHT",Right,-Bottom);
  Container:SetHitRectInsets(-Left,-Right,-Top,-Bottom);
end


--This function updates the parent containers for each bag, according to the options setup
function BaudUpdateJoinedBags()
  DebugMsg("Updating joined bags...");
  local OpenBags = {};
  for Bag = -1, LastBagID do
    OpenBags[Bag] = getglobal("BaudBagSubBag"..Bag):GetParent():IsShown();
    if OpenBags[Bag]then
      DebugMsg("Bag open: "..Bag);
    end
  end
  local SubBag, Level, Container, IsOpen, ContNum, BagID;
  for BagSet = 1, 2 do
    ContNum = 0;
    for Bag = 1, SetSize[BagSet]do
      if(Bag==1)or(BaudBag_Cfg[BagSet].Joined[Bag]==false)then
        if(Bag > 1)and not IsOpen then
          DebugMsg("Hiding Container "..Container:GetName());
          Container:Hide();
        elseif(Bag > 1)then
          DebugMsg("Showing Container "..Container:GetName());
          Container:Show();
        end
        IsOpen = false;
        ContNum = ContNum + 1;
        if(MaxCont[BagSet] < ContNum)then
          Container = CreateFrame("Frame","BBCont"..BagSet.."_"..ContNum,UIParent,"BaudBagContainerTemplate");
          Container:SetID(ContNum);
          Container.BagSet = BagSet;
          MaxCont[BagSet] = ContNum;
        end
        Container = getglobal("BBCont"..BagSet.."_"..ContNum);
        Container.Bags = {};
        BaudUpdateContainerData(BagSet,ContNum);
      end
      if(BagSet==1)then
        BagID = Bag - 1;
      elseif(Bag==1)then
        BagID = -1;
      else
        BagID = Bag + 3;
      end
      SubBag = getglobal("BaudBagSubBag"..BagID);
      tinsert(Container.Bags,SubBag);
      if(Bag > 1)then
        SubBag:SetParent(Container);
        --Setting the level of the sub container's buttons prevents them from sinking behind
        Level = SubBag:GetFrameLevel() + 1;
        for _, Frame in pairs({SubBag:GetChildren()})do
          Frame:SetFrameLevel(Level);
        end
      end
      if OpenBags[BagID]then
        IsOpen = true;
      end
    end
    if not IsOpen then
      DebugMsg("Hiding Container "..Container:GetName());
      Container:Hide();
    else
      DebugMsg("Showing Container "..Container:GetName());
      Container:Show();
    end
    NumCont[BagSet] = ContNum;
    for ContNum = (ContNum + 1), MaxCont[BagSet]do
      getglobal("BBCont"..BagSet.."_"..ContNum):Hide();
    end
  end
  BaudBag_UpdateBags();
end


function BaudBagUpdateOpenBags()
  local Open, Frame, Highlight;
  --The bank bag(-1) has no open indicator
  for Bag = 0, LastBagID do
    Frame = getglobal("BaudBagSubBag"..Bag);
    Open = Frame:IsShown()and Frame:GetParent():IsShown();
    if(Bag == 0)then
      MainMenuBarBackpackButton:SetChecked(Open);
    elseif(Bag <= 4)then
      getglobal("CharacterBag"..(Bag-1).."Slot"):SetChecked(Open);
    else
      Highlight = getglobal("BaudBBankBag"..(Bag-4).."HighlightFrameTexture");
      if Open then
        Highlight:Show();
      else
        Highlight:Hide();
      end
    end
  end
end


function BaudBagAutoOpenSet(BagSet)
  --Set 2 doesn't need container 1 to be shown because that's a given
  for ContNum = BagSet, NumCont[BagSet]do
    if BaudBag_Cfg[BagSet][ContNum].AutoOpen then
      getglobal("BBCont"..BagSet.."_"..ContNum):Show();
    end
  end
end


function BaudBagCloseBagSet(BagSet)
  for ContNum = 1, MaxCont[BagSet]do
    getglobal("BBCont"..BagSet.."_"..ContNum):Hide();
  end
end


local pre_ToggleBag = ToggleBag;
ToggleBag = function(id)
  if(id > 4)then
    if(BaudBag_Cfg[2].Enabled == false)then
      return pre_ToggleBag(id);
    end
  --The close button thing allows the original blizzard bags to be closed if they're still open
  elseif(BaudBag_Cfg[1].Enabled == false)or this and(strsub(this:GetName(),-11)=="CloseButton")then
    return pre_ToggleBag(id);
  end
  if(this==MailFrame)or(this==MerchantFrame)then
    BaudBagAutoOpenSet(1);
    return;
  end
  DebugMsg("Toggling bag: "..id.."("..(this and this:GetName() or "nil")..")");
  local Container = getglobal("BaudBagSubBag"..id);
  if not Container then
    return pre_ToggleBag(id);
  end
  Container = Container:GetParent();
  --if the bag to open is inside the main bank container, don't toggle it
  if(Container == BBCont2_1)then
    return;
  end
  if Container:IsShown() then
    Container:Hide();
  else
    Container:Show();
  end
end


local pre_OpenAllBags = OpenAllBags;
OpenAllBags = function(forceOpen)
  if(BaudBag_Cfg[1].Enabled == false)then
    return pre_OpenAllBags(forceOpen);
  end
  local Container, AnyShown;
  for Bag = 0, 4 do
    Container = getglobal("BaudBagSubBag"..Bag):GetParent();
    if not Container:IsShown() then
      Container:Show();
      AnyShown = true;
    end
  end
  if AnyShown then
    return;
  end
  BaudBagCloseBagSet(1);
end


local pre_BagSlotButton_OnClick = BagSlotButton_OnClick;
BagSlotButton_OnClick = function()
  if(BaudBag_Cfg[1].Enabled == false)then
    return pre_BagSlotButton_OnClick();
  end
  if CursorHasItem()then
    PutItemInBag(this:GetID());
    return;
  end
  ToggleBag(this:GetID() - CharacterBag0Slot:GetID() + 1);
end


local pre_BackpackButton_OnClick = BackpackButton_OnClick;
BackpackButton_OnClick = function()
  if(BaudBag_Cfg[1].Enabled == false)then
    return pre_BackpackButton_OnClick();
  end
  if CursorHasItem()then
    PutItemInBackpack();
    return;
  end
  ToggleBag(0);
end


--This is hooked to be able to replace the original bank box with this one
local pre_BankFrame_OnEvent = BankFrame_OnEvent;
BankFrame_OnEvent = function(event)
  if(BaudBag_Cfg[2].Enabled == false)then
    return pre_BankFrame_OnEvent(event);
  end
  if(event=="BANKFRAME_CLOSED")then
    BBCont2_1:Hide();

  elseif(event=="BANKFRAME_OPENED")then
    BBCont2_1:Show();
    BaudBagAutoOpenSet(1);
    BaudBagAutoOpenSet(2);
  end
end


function BaudBagSubBag_OnLoad()
  this:RegisterEvent("BAG_UPDATE");
  this:RegisterEvent("BAG_CLOSED");
  this:RegisterEvent("ITEM_LOCK_CHANGED");
  this:RegisterEvent("BAG_UPDATE_COOLDOWN");
  this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
end


function BaudBagUpdateColors(SubBag)
  local Link, Quality;
  local R, G, B;
  for Slot = 1, SubBag.size do
    R, G, B = 1, 1, 1;
    Link = GetContainerItemLink(SubBag:GetID(),Slot);
    if Link then
      Quality = select(3,GetItemInfo(Link));
      if Quality and(Quality > 1)then
        R, G, B = GetItemQualityColor(Quality);
      end
    end
    getglobal(SubBag:GetName().."Item"..Slot.."NormalTexture"):SetVertexColor(R, G, B, 1);
  end
end


function BaudBagSubBag_OnEvent()
  if not this:GetParent():IsShown()then
    return;
  end

  if(event=="BAG_UPDATE")and(this:GetID()==arg1)then
    --BAG_UPDATE is the only event called when a bag is added, so if no bag existed before, refresh
    if(this.size > 0)then
      ContainerFrame_Update(this);
      BaudBagUpdateColors(this);
    else
      this:GetParent().Refresh = true;
    end
    
  elseif(event=="ITEM_LOCK_CHANGED")or(event=="BAG_UPDATE_COOLDOWN")or(event=="UPDATE_INVENTORY_ALERTS")then
    ContainerFrame_Update(this);
    
  elseif(event=="BAG_CLOSED")and(this:GetID()==arg1)then
    --This event occurs when bags are swapped too, but updated information is not immediately
    --available to the addon, so the bag must be updated later.
    this:GetParent().Refresh = true;
  end
end


function BaudBagContainer_OnLoad()
  tinsert(UISpecialFrames,this:GetName());
  this:RegisterForDrag("LeftButton");
end


function BaudBagContainer_OnUpdate()
  if this.Refresh then
    this.Refresh = false;
    BaudBagUpdateContainer(this);
    BaudBagUpdateOpenBags();
  end
  if this.UpdateSlots then
    BaudBagUpdateFreeSlots(this);
  end
end


function BaudBagContainer_OnShow()
  PlaySound("igBackPackOpen");
  BaudBagUpdateContainer(this);
  if(this:GetID()==1)then
    BaudBagUpdateFreeSlots(this);
  end
  BaudBagUpdateOpenBags();
end


function BaudBagContainer_OnDragStart()
  if not BaudBag_Cfg[this.BagSet][this:GetID()].Locked then
    this:StartMoving();
  end
end


function BaudBagContainer_OnDragStop()
  this:StopMovingOrSizing();
  BaudBagContainerSaveCoords(this);
end


function BaudBagContainerSaveCoords(Frame)
  DebugMsg("Saving container coords: "..Frame:GetName());
  local Scale = Frame:GetScale();
  local X, Y = Frame:GetCenter();
  X = X * Scale;
  Y = Y * Scale;
  BaudBag_Cfg[Frame.BagSet][Frame:GetID()].Coords = {X, Y};
end


local TotalFree, TotalSlots;

local function AddFreeSlots(Bag)
  if(Bag > 0)then
    local Link = GetInventoryItemLink("player",ContainerIDToInventoryID(Bag));
    if not Link then
      return DebugMsg("No bag in slot "..Bag);
    end
    local Type = select(7, GetItemInfo(Link));
    if(Type~="Bag")then
      return DebugMsg("Not counting free slots in "..Bag.."("..Type..")");
    end
  end
  local NumSlots = GetContainerNumSlots(Bag);
  TotalSlots = TotalSlots + NumSlots;
  for Slot = 1, NumSlots do
    if not GetContainerItemInfo(Bag,Slot)then
      TotalFree = TotalFree + 1;
    end
  end
  return;
end


function BaudBagUpdateFreeSlots(Frame)
  DebugMsg("Counting free slots for set "..Frame.BagSet);
  TotalFree, TotalSlots = 0, 0;
  if(Frame.BagSet==1)then
    for Bag = 0, 4 do
      AddFreeSlots(Bag);
    end
  else
    AddFreeSlots(-1);
    for Bag = 5, LastBagID do
      AddFreeSlots(Bag);
    end
  end
  getglobal(Frame:GetName().."Slots"):SetText(TotalFree.."/"..TotalSlots.." Free");
  Frame.UpdateSlots = nil;
end


function BaudBagBankBags_OnLoad()
  this:RegisterEvent("PLAYER_MONEY");
  this:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED");
  this:SetBackdropColor(0,0,0,1);
  --Generate bank bag buttons for each bag slot
  local BagSlot;
  for Bag = 1, NUM_BANKBAGSLOTS do
    BagSlot = CreateFrame("Button","BaudBBankBag"..Bag,BaudBBank,"BankItemButtonBagTemplate");
    BagSlot:SetID(Bag + 4);
    BagSlot:SetPoint("TOPLEFT",8 + mod(Bag - 1, 2) * 39,-8 - floor((Bag - 1) / 2) * 39);
  end
  BaudBBank:SetWidth(91);
  BaudBBank.Height = 13 + ceil(NUM_BANKBAGSLOTS / 2) * 39;
  BaudBBank:SetHeight(BaudBBank.Height);
end


function BaudBagBankBags_OnEvent()
  if(event=="PLAYER_MONEY")or(event=="PLAYERBANKBAGSLOTS_CHANGED")then
    BaudBagBankBags_Update();
  end
end


function BaudBagBankBags_Update()
  local Purchase = BaudBBankPurchase;
  local Slots, Full = GetNumBankSlots();
  local BagSlot;

  for Bag = 1, NUM_BANKBAGSLOTS do
    BagSlot = getglobal("BaudBBankBag"..Bag);

    if(Bag <= Slots)then
      SetItemButtonTextureVertexColor(BagSlot, 1.0, 1.0, 1.0);
      BagSlot.tooltipText = BANK_BAG;
    else
      SetItemButtonTextureVertexColor(BagSlot, 1.0, 0.1, 0.1);
      BagSlot.tooltipText = BANK_BAG_PURCHASE;
    end
  end

  if Full then
    Purchase:Hide();
    BaudBBank:SetHeight(BaudBBank.Height);
    return;
  end

  local Cost = GetBankSlotCost(Slots);
  
  --This line allows the confirmation box to show the cost
  BankFrame.nextSlotCost = Cost;
  
  if( GetMoney() >= Cost ) then
    SetMoneyFrameColor(Purchase:GetName().."MoneyFrame", 1.0, 1.0, 1.0);
  else
    SetMoneyFrameColor(Purchase:GetName().."MoneyFrame", 1.0, 0.1, 0.1);
  end
  MoneyFrame_Update(Purchase:GetName().."MoneyFrame", Cost);

  Purchase:Show();
  BaudBBank:SetHeight(BaudBBank.Height + 40);
end


--This is for the button that toggles the bank bag display
function BaudBagBagsButton_OnClick()
  BaudBag_Cfg.BankBags = not BaudBag_Cfg.BankBags;
  BaudBagUpdateBankBags();
end


function BaudBagUpdateBankBags()
  local Shown = (BaudBag_Cfg.BankBags ~= false);
  BaudBagBagsButton:SetChecked(Shown);
  if Shown then
    BaudBBank:Show();
  else
    BaudBBank:Hide();
  end
end


--This function runs the update 
function BaudBag_UpdateBags()
  for BagSet = 1, 2 do
    for ContNum = 1, MaxCont[BagSet]do
      BaudBagUpdateContainer(getglobal("BBCont"..BagSet.."_"..ContNum));
    end
  end
end


function BaudBagUpdateContainer(Container)
  if not Container:IsShown()then
    return;
  end
  DebugMsg("Updating Container: "..Container:GetName());
  local Background = BaudBag_Cfg[Container.BagSet][Container:GetID()].Background;
  local MaxCols = BaudBag_Cfg[Container.BagSet][Container:GetID()].Columns;
  Container.Slots = 0;
  for _, SubBag in pairs(Container.Bags)do
    SubBag.size = GetContainerNumSlots(SubBag:GetID());
    Container.Slots = Container.Slots + SubBag.size;
  end
  
  local Col, Row = 0, 1;
  --The textured background puts its empty space on the upper left
--[[  if(Background==3)then
    Col = MaxCols - mod(Container.Slots - 1,MaxCols) - 1;
  end]]
  
  if(Container==BBCont2_1)then
    BaudBagBankBags_Update();
  end
  local Slots, SubBag, ItemButton;
  for _, SubBag in pairs(Container.Bags)do
    if(SubBag.size <= 0)then
      SubBag:Hide();
    else
      DebugMsg("Adding "..SubBag:GetName().." to "..Container:GetID());
      --Create extra slots if needed
      if(SubBag.size > (SubBag.maxSlots or 0))then
        for Slot = (SubBag.maxSlots or 0) + 1, SubBag.size do
          CreateFrame("Button",SubBag:GetName().."Item"..Slot,SubBag,(SubBag:GetName() ~= "BaudBagSubBag-1")and "ContainerFrameItemButtonTemplate" or "BankItemButtonGenericTemplate"):SetID(Slot);
        end
        SubBag.maxSlots = SubBag.size;
      end
      if(SubBag:GetID()>=0)then
        ContainerFrame_Update(SubBag);
      end
      BaudBagUpdateColors(SubBag);
      for Slot = 1, SubBag.maxSlots do
        ItemButton = getglobal(SubBag:GetName().."Item"..Slot);
        if(Slot<=SubBag.size)then
          Col = Col + 1;
          if(Col > MaxCols)then
            Col = 1;
            Row = Row + 1;
          end
          ItemButton:ClearAllPoints();
          --Slot spacing is different for the blizzard textured background
          if(Background<=3)then
            ItemButton:SetPoint("TOPLEFT",Container,"TOPLEFT",(Col-1)*42,(Row-1)*-41);
          else
            ItemButton:SetPoint("TOPLEFT",Container,"TOPLEFT",(Col-1)*39,(Row-1)*-39);
          end
          ItemButton:Show();
        else
          ItemButton:Hide();
        end
      end
      SubBag:Show();
    end
  end
  if(Background<=3)then
    Container:SetWidth(MaxCols * 42 - 5);
    Container:SetHeight(Row * 41 - 4);
  else
    Container:SetWidth(MaxCols * 39 - 2);
    Container:SetHeight(Row * 39 - 2);
  end
  BaudBagUpdateBackground(Container);
  DebugMsg("Finished Arranging Bag");
end

